# $Id: drilldownconfig.py 3017 2010-02-08 09:02:57Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-8.5.1/src/drilldownconfig.py $
import pprint
from merescocore.framework import Observable
import traceback

class DrilldownConfig(Observable):
    def __init__(self, infra):
        Observable.__init__(self)
        self.infra = infra
        self.logger = infra.logger
        self.makeDrilldownDict()

    def getDrilldownFields(self,name):
        resultList = []

        drilldownConfig = self.infra.configs['drilldown']
        serverConfig    = self.infra.configs['server']
        fieldsList = drilldownConfig.getList(name, section = "drilldownFields")

        for name in fieldsList:
            if name == '':
                continue
            if not name in self.fieldFeatures:
                s  = 'Drilldownconfig ERROR: "' + drilldownConfig.configFile +':[drilldownFields]:useDrilldownFields"\n'
                s += 'contains invalid field: "' + name + '"\n'
                s += 'This should match config in: "' + serverConfig.configFile + ':[indices]:drilldownFieldnames"\n'
                self.logger.error(s)
                continue
                tupe = drilldownConfig.get('drillDownFieldsDefaults', default = '10 5 True', section = 'drilldownFields')

            else:
                tupe = self.fieldFeatures[name]

            (nrRequested, nrDisplayed, boolean) = tupe
            resultList.append ( (name, nrRequested, boolean) )

        return resultList

    def getFieldFeatures(self):
        return self.fieldFeatures

    def getMultiLevelDrilldown(self):
        drilldownConfig = self.infra.configs['drilldown']
        serverConfig    = self.infra.configs['server']
        drillDict = drilldownConfig.getSectionDict('multilevelDrilldown')
        cleanDict = {}

        for level in drillDict:
            values = drillDict[level].split('\n')
            fields = []
            for field in values:
                if field == '':
                    continue

                if not field in self.fieldFeatures:
                    s  = 'Drilldownconfig ERROR: "' + drilldownConfig.configFile +':[multilevelDrilldown]"\n'
                    s += 'contains invalid field: "' + field + '"\n'
                    s += 'This should match config in: "' + serverConfig.configFile + ':[untokenizedFields]:drilldownFieldnames"\n'
                    self.logger.error(s)
                    continue
                fields.append(field)

            cleanDict [level] = fields

#        self.logger.debug(self.infra.prpt("cleanDict",cleanDict))
        return cleanDict

    def makeDrilldownDict(self):
        drilldownConfig = self.infra.configs['drilldown']
        serverConfig    = self.infra.configs['server']

        fieldFeatures = {}
        drilldownDefaults = drilldownConfig.get('drillDownFieldsDefaults', default = '10 5 True', section = 'drilldownFields')
        (defaultNrRequested, defaultNrDisplayed, defaultBoolean) = drilldownDefaults.split()

        allowedFieldsList = serverConfig.getList('drilldownFieldnames', section = 'untokenizedFields')
        useFieldsList = drilldownConfig.getList('useDrilldownFields', section = 'drilldownFields')

        list = []
        for line in useFieldsList:
            if line == "":
                continue
            nameList = line.split()

            name = nameList[0]
            name = name.strip()

            if not name in allowedFieldsList:
                s  = 'Drilldownconfig ERROR: "' + drilldownConfig.configFile +':[drilldownFields]:useDrilldownFields"\n'
                s += 'contains invalid field: "' + name + '"\n'
                s += 'This should match config in: "' + serverConfig.configFile + ':[untokenizedFields]:drilldownFieldnames"\n'
                self.logger.error(s)
                continue

            boolean = defaultBoolean
            nrRequested = defaultNrRequested
            nrDisplayed = defaultNrDisplayed
            if len (nameList) == 1:
                nrRequested = defaultNrRequested
                nrDisplayed = defaultNrDisplayed
                boolean = defaultBoolean
            elif len (nameList) == 4:
                # ternary expressions in python :(
                nrRequested = nameList[1] if nameList[1] else defaultNrRequested
                nrDisplayed = nameList[2] if nameList[2] else defaultNrDisplayed
                boolean     = nameList[3] if nameList[3] else defaultBoolean

            if boolean == 'True':
                boolean = True
            else:
                boolean = False

            nrRequested = int (nrRequested)
            nrDisplayed = int(nrDisplayed)

            self.logger.debug('name: ' + name + ' --nrRequested: ' + str(nrRequested)  )
            tupe = (name, nrRequested, boolean)
            list.append(tupe)

            if (nrRequested and nrDisplayed > 0 and nrDisplayed > nrRequested):
                self.logger.warning ('Number of displayed items (' + str(nrDisplayed) + ') exceeds number of requested items ('+ str(nrRequested) +') for "' + name + '"')
                nrDisplayed = nrRequested

            fieldFeatures[ name ] = ( nrRequested, nrDisplayed, boolean)

        self.fieldFeatures = fieldFeatures
        return fieldFeatures
